@@ -10,10 +10,10 @@ from jax import random
 from .basic_manifold_jax import basic_manifold_jax
 
 class sphere_jax(basic_manifold_jax):
-    def __init__(self, var_shape, train_mode = False) -> None:
-
+    def __init__(self, var_shape, train_mode = False, device = None) -> None:
+        self.device = device
         if not train_mode:
-            super().__init__('sphere',var_shape, (1,))
+            super().__init__('sphere',var_shape, (1,), device=self.device)
     
 
 
@@ -72,7 +72,7 @@ class sphere_jax(basic_manifold_jax):
         
         X_rvec = jnp.sqrt(jnp.sum( Xinit * Xinit ))
         Xinit = Xinit / X_rvec
-        
+        Xinit = jax.device_put(Xinit, self.device)
         return Xinit
 
     def Post_process(self,X):
