@@ -5,14 +5,11 @@ from .basic_manifold_np import basic_manifold_np
 
 class sphere_np(basic_manifold_np):
     def __init__(self, var_shape) -> None:
-        n = var_shape[0]
-        p = var_shape[1]
-        self._n = n
-        self._p = p
+        
 
 
-        super().__init__('Sphere',(n,p), (1,))
-    
+        super().__init__('Sphere',var_shape, (1,))
+
 
 
     def A(self, X):
@@ -61,7 +58,7 @@ class sphere_np(basic_manifold_np):
 
     def Init_point(self, Xinit = None):
         if Xinit is None:
-            Xinit = np.random.randn(self._n, self._p)
+            Xinit = np.random.randn(self.var_shape)
         
         X_rvec = np.sqrt(np.sum( Xinit * Xinit ))
         Xinit = Xinit / X_rvec
