@@ -88,7 +88,7 @@ class _ConvNd_cdopt(nn.Module):
                  bias: bool,
                  padding_mode: str,
                  device=None,
-                 dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = wvt_conv_flatten, manifold_args = {}) -> None:
+                 dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = wvt_conv_flatten, manifold_args = {}, forward_with_A:bool = True) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(_ConvNd_cdopt, self).__init__()
         if in_channels % groups != 0:
@@ -119,6 +119,7 @@ class _ConvNd_cdopt(nn.Module):
         self.groups = groups
         self.padding_mode = padding_mode
         self.penalty_param = penalty_param
+        self.forward_with_A = forward_with_A
         # `_reversed_padding_repeated_twice` is the padding to be passed to
         # `F.pad` if needed (e.g., for non-zero padding types that are
         # implemented as two ops: padding + conv). `F.pad` accepts paddings in
@@ -228,7 +229,7 @@ class Conv1d_cdopt(_ConvNd_cdopt):
         padding_mode: str = 'zeros',  # TODO: refine this type
         device=None,
         dtype=None, manifold_class = euclidean_torch, penalty_param = 0,  
-        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}, forward_with_A:bool = True
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         # we create new variables below to make mypy happy since kernel_size has
@@ -241,7 +242,7 @@ class Conv1d_cdopt(_ConvNd_cdopt):
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _single(0), groups, bias, padding_mode, **factory_kwargs, 
             manifold_class = manifold_class, penalty_param=penalty_param,  
-            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args, forward_with_A=forward_with_A)
 
 
 
@@ -254,7 +255,10 @@ class Conv1d_cdopt(_ConvNd_cdopt):
                         self.padding, self.dilation, self.groups)
 
     def forward(self, input: Tensor) -> Tensor:
-        return self._conv_forward(input, self.A(self.weight), self.bias)
+        if self.forward_with_A:
+            return self._conv_forward(input, self.A(self.weight), self.bias)
+        else:
+            return self._conv_forward(input, self.weight, self.bias)
 
 
 
@@ -274,7 +278,7 @@ class Conv2d_cdopt(_ConvNd_cdopt):
         padding_mode: str = 'zeros',  # TODO: refine this type
         device=None,
         dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
-        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}, forward_with_A:bool = True
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size_ = _pair(kernel_size)
@@ -285,7 +289,7 @@ class Conv2d_cdopt(_ConvNd_cdopt):
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _pair(0), groups, bias, padding_mode, **factory_kwargs, 
             manifold_class = manifold_class, penalty_param = penalty_param,  
-            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args, forward_with_A=forward_with_A)
 
     def _conv_forward(self, input: Tensor, weight: Tensor, bias: Optional[Tensor]):
         if self.padding_mode != 'zeros':
@@ -296,7 +300,10 @@ class Conv2d_cdopt(_ConvNd_cdopt):
                         self.padding, self.dilation, self.groups)
 
     def forward(self, input: Tensor) -> Tensor:
-        return self._conv_forward(input, self.A(self.weight), self.bias)
+        if self.forward_with_A:
+            return self._conv_forward(input, self.A(self.weight), self.bias)
+        else:
+            return self._conv_forward(input, self.weight, self.bias)
 
 
 
@@ -315,7 +322,7 @@ class Conv3d_cdopt(_ConvNd_cdopt):
         padding_mode: str = 'zeros',
         device=None,
         dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
-        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}, forward_with_A:bool = True
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size_ = _triple(kernel_size)
@@ -326,7 +333,7 @@ class Conv3d_cdopt(_ConvNd_cdopt):
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _triple(0), groups, bias, padding_mode, **factory_kwargs, 
             manifold_class = manifold_class, penalty_param = penalty_param,  
-            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args, forward_with_A= forward_with_A)
 
     def _conv_forward(self, input: Tensor, weight: Tensor, bias: Optional[Tensor]):
         if self.padding_mode != "zeros":
@@ -346,7 +353,10 @@ class Conv3d_cdopt(_ConvNd_cdopt):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        return self._conv_forward(input, self.A(self.weight), self.bias)
+        if self.forward_with_A:
+            return self._conv_forward(input, self.A(self.weight), self.bias)
+        else:
+            return self._conv_forward(input, self.weight, self.bias)
 
 
 
