@@ -24,7 +24,7 @@ class Linear_cdopt(nn.Module):
     weight: Tensor
 
     def __init__(self, in_features: int, out_features: int, bias: bool = True,
-                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, manifold_args = {}) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param: float = 0, manifold_args = {}, forward_with_A:bool = True) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(Linear_cdopt, self).__init__()
         self.in_features = in_features
@@ -32,6 +32,7 @@ class Linear_cdopt(nn.Module):
         self.manifold_class = manifold_class
         self.manifold_args = manifold_args
         self.penalty_param = penalty_param
+        self.forward_with_A = forward_with_A
 
         if self.in_features >= self.out_features:
             self.manifold = manifold_class((self.in_features, self.out_features), device= device, dtype= dtype, **manifold_args)
@@ -102,7 +103,10 @@ class Linear_cdopt(nn.Module):
         self.quad_penalty = lambda :  torch.sum(self.C(self.weight)**2)
 
     def forward(self, input: Tensor) -> Tensor:
-        return F.linear(input, self.A(self.weight), self.bias)
+        if self.forward_with_A :
+            return F.linear(input, self.A(self.weight), self.bias)
+        else:
+            return F.linear(input, self.weight, self.bias)
 
     def extra_repr(self) -> str:
         return 'in_features={}, out_features={}, bias={}'.format(
@@ -123,7 +127,7 @@ class Bilinear_cdopt(nn.Module):
     weight: Tensor
 
     def __init__(self, in1_features: int, in2_features: int, out_features: int, bias: bool = True, 
-                 device=None, dtype=None, manifold_class = euclidean_torch,penalty_param = 0, weight_var_transfer = None, manifold_args = {}) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch,penalty_param = 0, weight_var_transfer = None, manifold_args = {}, forward_with_A:bool = True) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(Bilinear_cdopt, self).__init__()
         self.in1_features = in1_features
@@ -132,6 +136,7 @@ class Bilinear_cdopt(nn.Module):
 
         self.manifold_args = manifold_args
         self.penalty_param = penalty_param
+        self.forward_with_A = forward_with_A
 
 
         if weight_var_transfer is None:
@@ -190,7 +195,10 @@ class Bilinear_cdopt(nn.Module):
             init.uniform_(self.bias, -bound, bound)
 
     def forward(self, input1: Tensor, input2: Tensor) -> Tensor:
-        return F.bilinear(input1, input2, self.A(self.weight), self.bias)
+        if self.forward_with_A:
+            return F.bilinear(input1, input2, self.A(self.weight), self.bias)
+        else:
+            return F.bilinear(input1, input2, self.weight, self.bias)
 
     def extra_repr(self) -> str:
         return 'in1_features={}, in2_features={}, out_features={}, bias={}'.format(
